;;; site-lisp.el --- Manage site-lisp directories  -*- lexical-binding: t; -*-

;; Copyright (C) 2021  Philip Kaludercic

;; Author: Philip Kaludercic <philipk@posteo.net>
;; Keywords: lisp, local

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:

;; The following code manages site-local Lisp code, that might not be
;; part of a package.  For regular use, create a "site-lisp" directory
;; next to "init.el", and create a subdirectory for every program you
;; wish to use.

;;; Code:

(defgroup site-lisp ()
  "Manage site-lisp directories."
  :group 'lisp)

(defcustom site-lisp-autoload-file "autoload.el"
  "Name of file to store autoload forms in."
  :type 'string
  :version "28.1")

(defun site-lisp-prepare (dir)
  "Byte-compile, prepare autoloads and note each directory in DIR."
  (interactive "DPrepare: ")
  (let* ((backup-inhibited t)
         (autoload-file (expand-file-name site-lisp-autoload-file dir)))
    (dolist (dir (directory-files dir t "^[^.]"))
      (when (file-directory-p dir)
        (add-to-list 'load-path dir)
        (make-directory-autoloads dir autoload-file)
        (byte-recompile-directory dir)))
    (add-to-list 'load-path dir)
    (load autoload-file nil t)))

(defun site-lisp-unprepare (dir)
  "Remove every directory in DIR from `load-path'."
  (interactive "DUnprepare: ")
  (dolist (dir (directory-files dir t "^[^.]"))
    ;; TODO: reverse everything that might have been autoloaded.
    (when (cons dir (file-directory-p dir))
      (setq load-path (remove dir load-path)))))

;;;###autoload
(defcustom site-lisp-directory
  (locate-user-emacs-file "site-lisp")
  "Directory use for site-local Lisp code.
If this directory doesn't exist, nothing is done."
  :set (lambda (var val)
	 (when (and (boundp 'site-lisp-directory)
                    (file-directory-p site-lisp-directory))
	   (site-lisp-unprepare site-lisp-directory))
	 (when (file-directory-p val)
	   (site-lisp-prepare val))
	 (set var val))
  :type 'directory
  :version "28.1")

;;;###autoload
(defun site-lisp-reload ()
  "Reload the contents of `site-lisp-directory'."
  (interactive)
  (site-lisp-prepare site-lisp-directory))

(provide 'site-lisp)
;;; site-lisp.el ends here
